@@ -1,6 +1,6 @@
 /*
  * ProFTPD - mod_sftp 'keyboard-interactive' user authentication
- * Copyright (c) 2008-2014 TJ Saunders
+ * Copyright (c) 2008-2015 TJ Saunders
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,8 +20,6 @@
  * give permission to link this program with OpenSSL, and distribute the
  * resulting executable, without including the source code for OpenSSL in the
  * source distribution.
- *
- * $Id: auth-kbdint.c,v 1.10 2014/03/02 22:05:43 castaglia Exp $
  */
 
 #include "mod_sftp.h"
@@ -34,6 +32,14 @@
 #include "utf8.h"
 #include "kbdint.h"
 
+/* This array tracks the names of kbdint drivers that have successfully
+ * authenticated the user.  In any given session, the same kbdint driver CANNOT
+ * be used twice for authentication; this supports authentication chains
+ * like "keyboard-interactive+keyboard-interactive", requiring clients to
+ * authenticate using multiple different kbdint means.
+ */
+static array_header *kbdint_drivers = NULL;
+
 static const char *trace_channel = "ssh2";
 
 int sftp_auth_kbdint(struct ssh2_packet *pkt, cmd_rec *pass_cmd,
@@ -50,6 +56,10 @@ int sftp_auth_kbdint(struct ssh2_packet *pkt, cmd_rec *pass_cmd,
       "no 'keyboard-interactive' drivers currently registered, unable to "
       "authenticate user '%s' via 'keyboard-interactive' method", user);
 
+    pr_log_auth(PR_LOG_NOTICE,
+      "USER %s (Login failed): keyboard-interactive authentication disabled",
+      user);
+
     *send_userauth_fail = TRUE;
     errno = EPERM;
     return 0;
@@ -58,7 +68,11 @@ int sftp_auth_kbdint(struct ssh2_packet *pkt, cmd_rec *pass_cmd,
   if (pr_cmd_dispatch_phase(pass_cmd, PRE_CMD, 0) < 0) {
     (void) pr_log_writefile(sftp_logfd, MOD_SFTP_VERSION,
       "authentication request for user '%s' blocked by '%s' handler",
-      orig_user, pass_cmd->argv[0]);
+      orig_user, (char *) pass_cmd->argv[0]);
+
+    pr_log_auth(PR_LOG_NOTICE,
+      "USER %s (Login failed): blocked by '%s' handler", orig_user,
+      (char *) pass_cmd->argv[0]);
 
     pr_cmd_dispatch_phase(pass_cmd, POST_CMD_ERR, 0);
     pr_cmd_dispatch_phase(pass_cmd, LOG_CMD_ERR, 0);
@@ -103,6 +117,12 @@ int sftp_auth_kbdint(struct ssh2_packet *pkt, cmd_rec *pass_cmd,
         "cipher algorithm '%s' or MAC algorithm '%s' unacceptable for "
         "keyboard-interactive authentication, denying authentication request",
         cipher_algo, mac_algo);
+
+      pr_log_auth(PR_LOG_NOTICE,
+        "USER %s (Login failed): cipher algorithm '%s' or MAC algorithm '%s' "
+        "unsupported for keyboard-interactive authentication", user,
+        cipher_algo, mac_algo);
+
       *send_userauth_fail = TRUE;
       errno = EPERM;
       return 0;
@@ -125,9 +145,31 @@ int sftp_auth_kbdint(struct ssh2_packet *pkt, cmd_rec *pass_cmd,
    */
 
   driver = sftp_kbdint_first_driver();
-  while (driver) {
+  while (driver != NULL) {
+    register unsigned int i;
+    int skip_driver = FALSE;
+
     pr_signals_handle();
 
+    /* If this driver has already successfully handled this user, skip it. */
+    for (i = 0; i < kbdint_drivers->nelts; i++) {
+      char *dri;
+
+      dri = ((char **) kbdint_drivers->elts)[i];
+      if (strcmp(driver->driver_name, dri) == 0) {
+        skip_driver = TRUE;
+        break;
+      }
+    }
+
+    if (skip_driver) {
+      pr_trace_msg(trace_channel, 9,
+        "skipping already-used kbdint driver '%s' for user '%s'",
+        driver->driver_name, user);
+      driver = sftp_kbdint_next_driver();
+      continue;
+    }
+
     pr_trace_msg(trace_channel, 3, "trying kbdint driver '%s' for user '%s'",
       driver->driver_name, user);
 
@@ -140,8 +182,12 @@ int sftp_auth_kbdint(struct ssh2_packet *pkt, cmd_rec *pass_cmd,
     res = driver->authenticate(driver, user);
     driver->close(driver);
 
-    if (res == 0)
+    if (res == 0) {
+      /* Store the driver name for future checking. */
+      *((char **) push_array(kbdint_drivers)) = pstrdup(sftp_pool,
+        driver->driver_name);
       break; 
+    }
 
     driver = sftp_kbdint_next_driver();
   }
@@ -160,3 +206,9 @@ int sftp_auth_kbdint(struct ssh2_packet *pkt, cmd_rec *pass_cmd,
 
   return 1;
 }
+
+int sftp_auth_kbdint_init(pool *p) {
+  kbdint_drivers = make_array(p, 0, sizeof(char *));
+
+  return 0;
+}
