@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2001-2019 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
  *
  * As a special exemption, Public Flood Software/MacGyver aka Habeeb J. Dihu
  * and other respective copyright holders give permission to link this program
@@ -24,31 +24,11 @@
  * the source code for OpenSSL in the source distribution.
  */
 
-/* User configurable defaults and tunable parameters.
- *
- * $Id: options.h,v 1.18 2003/06/04 23:32:02 castaglia Exp $
- */
+/* User configurable defaults and tunable parameters. */
 
 #ifndef PR_OPTIONS_H
 #define PR_OPTIONS_H
 
-/* Define the next option if your libc needs persistant /etc/passwd
- * and /etc/group functions.  Some libcs occasionally close these files
- * which can not be re-opened after a chroot().  Symptoms of this
- * include the inability to see user/group names when doing a 'ls -l' from
- * an anon. ftp login (you see only uid/gid numbers).
- */
-
-/* If we have setpassent(), NEED_PERSISTENT_PASSWD is not enabled
- * by default.  This option controls the DEFAULT value of the
- * PersistentPasswd directive.  You can always override this in
- * the configuration file.
- */
-
-#if ! (defined (HAVE_SETPASSENT) || defined (HAVE__PW_STAYOPEN))
-# define NEED_PERSISTENT_PASSWD
-#endif
-
 /* Tunable parameters */
 
 /* This defines the timeout for the main select() loop, defines the number
@@ -68,34 +48,70 @@
  * burst before the kernel rejects.  This can be configured by the
  * "tcpBackLog" configuration directive, this value is just the default.
  */
+#ifndef PR_TUNABLE_DEFAULT_BACKLOG
+# define PR_TUNABLE_DEFAULT_BACKLOG	128
+#endif /* PR_TUNABLE_DEFAULT_BACKLOG */
 
-#define PR_TUNABLE_DEFAULT_BACKLOG	5
-
-/* The next two define the default receive/send tcp windows (and
- * internal ProFTPD buffer sizes.  These can be configured per server
- * or per virtual-server via the tcpReceiveWindow and tcpSendWindow
- * directives.
+/* The default TCP send/receive buffer sizes, should explicit sizes not
+ * be defined at compile time, or should the runtime determination process
+ * fail.
+ *
+ * Note that these default buffer sizes are only used if the service cannot
+ * determine the platform's favored network buffer sizes using getsockopt(2).
+ * If you wish to override the use of getsockopt(2) to determine the network
+ * buffer sizes to use, you can use the PR_TUNABLE_RCVBUFSZ and
+ * PR_TUNABLE_SNDBUFSZ macros to define, at compile-time, the network buffer
+ * sizes to use.
  */
 
-#define PR_TUNABLE_DEFAULT_RCVBUF	8192
-#define PR_TUNABLE_DEFAULT_SNDBUF	8192
+#ifndef PR_TUNABLE_DEFAULT_RCVBUFSZ
+# define PR_TUNABLE_DEFAULT_RCVBUFSZ	8192
+#endif /* PR_TUNABLE_DEFAULT_RCVBUFSZ */
+
+#ifndef PR_TUNABLE_DEFAULT_SNDBUFSZ
+# define PR_TUNABLE_DEFAULT_SNDBUFSZ	8192
+#endif /* PR_TUNABLE_DEFAULT_SNDBUFSZ */
 
 /* Default internal buffer size used for data transfers and other
  * miscellaneous tasks.
  */
 #ifndef PR_TUNABLE_BUFFER_SIZE
-# define PR_TUNABLE_BUFFER_SIZE	1024
+# define PR_TUNABLE_BUFFER_SIZE		1024
 #endif
 
-/* This one is specifically for data transfers.
+/* There is also a definable buffer size used specifically for parsing
+ * lines of text from the config file: PR_TUNABLE_PARSER_BUFFER_SIZE.
+ *
+ * You should manually set the PR_TUNABLE_PARSER_BUFFER_SIZE only if you
+ * have exceptionally long configuration lines.
+ */
+#ifndef PR_TUNABLE_PARSER_BUFFER_SIZE
+# define PR_TUNABLE_PARSER_BUFFER_SIZE	4096
+#endif
+
+/* There is also a definable buffer size used specifically for data
+ * transfers: PR_TUNABLE_XFER_BUFFER_SIZE.  By default, this buffer
+ * size is automatically determined, at runtime, as the smaller of the
+ * TCP receive and send buffer sizes.
+ *
+ * You should manually set the PR_TUNABLE_XFER_BUFFER_SIZE only in
+ * special circumstances, when you need to explicitly control that
+ * buffer size.
  */
 #ifndef PR_TUNABLE_XFER_BUFFER_SIZE
 # define PR_TUNABLE_XFER_BUFFER_SIZE	PR_TUNABLE_BUFFER_SIZE
 #endif
 
+/* Maximum FTP command size.  For details on this size of 512KB, see
+ * the Bug#4014 discussion.
+ */
+#ifndef PR_TUNABLE_CMD_BUFFER_SIZE
+# define PR_TUNABLE_CMD_BUFFER_SIZE	(512 * 1024)
+#endif
+
 /* Maximum path length.  GNU HURD (and some others) do not define
  * MAXPATHLEN.  POSIX' PATH_MAX is mandated to be at least 256 
- * (according to some), so 1K, in the absense of MAXPATHLEN, should be
+ * (according to some), so 1K, in the absence of MAXPATHLEN, should be
  * a reasonable default.
  */
 
@@ -119,8 +135,14 @@
 # define PR_TUNABLE_TIMEOUTIDLE		600
 #endif
 
+/* The default command timeout in many command-line FTP clients (e.g.
+ * lukemftp, used on BSDs and maybe Linux?) is 60 seconds.  To avoid having
+ * those clients close the control connection because proftpd takes too
+ * long, while performing lingering closes, to send a response, keep the
+ * default linger timeout under 60 seconds.
+ */
 #ifndef PR_TUNABLE_TIMEOUTLINGER
-# define PR_TUNABLE_TIMEOUTLINGER	180
+# define PR_TUNABLE_TIMEOUTLINGER	10
 #endif
 
 #ifndef PR_TUNABLE_TIMEOUTLOGIN
@@ -162,18 +184,42 @@
 # define PR_TUNABLE_SCOREBOARD_SCRUB_TIMER	30
 #endif
 
-/* Max number of recursion/directory levels to support when globbing.
+/* Maximum number of attempted updates to the scoreboard during a
+ * file transfer before an actual write is done.  This is to allow
+ * an optimization where the scoreboard is not updated on every loop
+ * through the transfer buffer.
  */
 
-#ifndef PR_TUNABLE_GLOBBING_MAX	
-# define PR_TUNABLE_GLOBBING_MAX	8
+#ifndef PR_TUNABLE_XFER_SCOREBOARD_UPDATES
+# define PR_TUNABLE_XFER_SCOREBOARD_UPDATES	10
 #endif
 
-/* Maximum length of login name.
+#ifndef PR_TUNABLE_CALLER_DEPTH
+/* Max depth of call stack if stacktrace support is enabled. */
+# define PR_TUNABLE_CALLER_DEPTH	32
+#endif
+
+#ifndef PR_TUNABLE_ENV_MAX
+/* Max length of environment variable values allowed by proftpd. */
+# define PR_TUNABLE_ENV_MAX			2048
+#endif
+
+#ifndef PR_TUNABLE_GLOBBING_MAX_RECURSION
+/* Max number of recursion/directory levels to support when globbing.
  */
+# define PR_TUNABLE_GLOBBING_MAX_RECURSION	8
+#endif
+
+#ifndef PR_TUNABLE_GLOBBING_MAX_MATCHES
+/* Max number of matches to support when globbing.
+ */
+# define PR_TUNABLE_GLOBBING_MAX_MATCHES	100000UL
+#endif
 
 #ifndef PR_TUNABLE_LOGIN_MAX
-/* Ideally, we'd use _POSIX_LOGIN_NAME_MAX here, if it was defined.  However,
+/* Maximum length of login name.
+ *
+ * Ideally, we'd use _POSIX_LOGIN_NAME_MAX here, if it was defined.  However,
  * doing so would cause trouble for those sites that use databases for
  * storing user information; such sites often use email addresses as
  * login names.  Given that, let's use 256 as a login name size.
@@ -181,12 +227,31 @@
 # define PR_TUNABLE_LOGIN_MAX		256
 #endif
 
-/* Loopback network, this should generally not need to be changed,
- * although you can set a specific address by setting it to say
- * "127.0.0.1" and the netmask to "255.255.255.255".
- */
+#ifndef PR_TUNABLE_PASSWORD_MAX
+/* Maximum length of a password. */
+# define PR_TUNABLE_PASSWORD_MAX	1024
+#endif
 
-#define LOOPBACK_NET            "127.0.0.0"
-#define LOOPBACK_MASK           "255.255.255.0"
+#ifndef PR_TUNABLE_EINTR_RETRY_INTERVAL
+/* Define the time to delay, in seconds, after a system call has been
+ * interrupted (errno is EINTR) before retrying that call.
+ *
+ * The default behavior is delay 0.2 secs between retries.
+ */
+# define PR_TUNABLE_EINTR_RETRY_INTERVAL	0.2
+#endif
+
+#ifndef PR_TUNABLE_XFER_LOG_MODE
+# define PR_TUNABLE_XFER_LOG_MODE		0644
+#endif
+
+/* FS Statcache tuning. */
+#ifndef PR_TUNABLE_FS_STATCACHE_SIZE
+# define PR_TUNABLE_FS_STATCACHE_SIZE		30000
+#endif
+
+#ifndef PR_TUNABLE_FS_STATCACHE_MAX_AGE
+# define PR_TUNABLE_FS_STATCACHE_MAX_AGE	3
+#endif
 
 #endif /* PR_OPTIONS_H */
